package main

import (
	"encoding/json"
	"fmt"
	"log/slog"

	mqtt "github.com/eclipse/paho.mqtt.golang"
)

// MQTTSetup creates a new MQTT client
func MQTTSetup(cfg *Config, publishHandler mqtt.MessageHandler) mqtt.Client {
	// This JS style of creating a client is awfully verbose
	opts := mqtt.NewClientOptions().
		AddBroker(fmt.Sprintf("tcp://%s:%d", cfg.brokerHost, cfg.brokerPort)).
		SetClientID(mqttClientID).
		SetDefaultPublishHandler(publishHandler).SetAutoReconnect(true)

	client := mqtt.NewClient(opts)
	if token := client.Connect(); token.Wait() && token.Error() != nil {
		panic(token.Error())
	}
	slog.Debug("Connected to", "Broker", cfg.brokerHost, "Port", cfg.brokerPort, "Client", mqttClientID)
	if token := client.Subscribe(cfg.topic+"/#", 0, nil); token.Wait() && token.Error() != nil {
		panic(token.Error())
	}
	return client
}

// MQTTDumpMsg displays the MQTT message
func MQTTDumpMsg(msg mqtt.Message) {
	var data map[string]interface{}
	err := json.Unmarshal(msg.Payload(), &data)
	if err != nil {
		slog.Error("Error decoding JSON %v", "topic", msg.Topic(), "err", err)
	}
	jsonDump(data)
}
